
#include <stdio.h>
#include <stdlib.h>

void cdft(int, int, double *);

// complex multiplication
// (x + yi)(u + vi) = (xu - yv) + (xv + yu)i.
void cmul(int nfft, double in[], double inout[])
{
    int i;
    double x, y, u, v;
    for(i=0; i < nfft; i+=2)
    {
      x = in[i];
      y = in[i+1];
      u = inout[i];
      v = inout[i+1];
      inout[i] = x*u - y*v;
      inout[i+1] = x*v + y*u;
    }        
}

int main(int argc, char **argv)
{
  const int n = 4, radix = 256;  // n == fft size, radix is (obviously) radix used - in this case, byte sized
  double a[4*2], b[4*2], c[4*2*2];
  int result;
  int i, carry, help;
  int n1, n2;
  
  if(argc != 3)
  {
    printf("Usage: %s n1 n2\n", argv[0]);
    return 0;
  }
  
  // initialize vars
  for(i=0; i < 4*2; i++)
    a[i] = b[i] = c[i] = c[i+n*2] = 0.0;
  n1 = atoi(argv[1]);
  n2 = atoi(argv[2]);
  
  // Convert numbers to polynomial form - using real part only
  i=0;  
  while(n1)
  {
    a[i*2] = n1 % radix;
    n1 /= radix;
    i++;
  }
  
  i=0;  
  while(n2)
  {
    b[i*2] = n2 % radix;
    n2 /= radix;
    i++;
  }
  
  // Forward FFT both numbers  
  cdft(2*n, 1, a);  
  cdft(2*n, 1, b);
  
  // Multiply the arrays dyadically
  cmul(2*n, b, a);
  
  // Inverse FFT of result
  cdft(2*n, -1, a);
  // Normalize (this FFT code is gay)
  for(i=0; i < 2*n; i++)
    a[i] *= 1.0 / n;
    
  // now evaluate the polynomial to get our integer back
  // this is pretty much just big number addition with carry
  carry = result = 0;
  help = 1;
  for(i=0; i < n*2; i+=2)
  {
    result += (((int)a[i] + carry) % radix) * help;
    carry = a[i] / radix;
    help *= radix;
  }
  result += carry;
  
  // ta-da
  printf("%d\n", result);
  
  return 0;  
}

